using System;
using System.Net;
using System.Data;
using System.Linq;
using Microsoft.Data.SqlClient;
using System.Collections.Generic;
using Newtonsoft.Json.Linq;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNet.OData;
using Microsoft.AspNet.OData.Routing;
using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Metadata.Internal;
using Microsoft.AspNet.OData.Query;



namespace SupAliAdminSite.Controllers.SupAliDb
{
  using Models;
  using Data;
  using Models.SupAliDb;

  [ODataRoutePrefix("odata/SupAliDB/SupaliCustodians")]
  [Route("mvc/odata/SupAliDB/SupaliCustodians")]
  public partial class SupaliCustodiansController : ODataController
  {
    private Data.SupAliDbContext context;

    public SupaliCustodiansController(Data.SupAliDbContext context)
    {
      this.context = context;
    }
    // GET /odata/SupAliDb/SupaliCustodians
    [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
    [HttpGet]
    public IEnumerable<Models.SupAliDb.SupaliCustodian> GetSupaliCustodians()
    {
      var items = this.context.SupaliCustodians.AsQueryable<Models.SupAliDb.SupaliCustodian>();
      this.OnSupaliCustodiansRead(ref items);

      return items;
    }

    partial void OnSupaliCustodiansRead(ref IQueryable<Models.SupAliDb.SupaliCustodian> items);

    [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
    [HttpGet("{Custodian_ID}")]
    public SingleResult<SupaliCustodian> GetSupaliCustodian(int key)
    {
        var items = this.context.SupaliCustodians.Where(i=>i.Custodian_ID == key);
        this.OnSupaliCustodiansGet(ref items);

        return SingleResult.Create(items);
    }

    partial void OnSupaliCustodiansGet(ref IQueryable<Models.SupAliDb.SupaliCustodian> items);

    partial void OnSupaliCustodianDeleted(Models.SupAliDb.SupaliCustodian item);

    [HttpDelete("{Custodian_ID}")]
    public IActionResult DeleteSupaliCustodian(int key)
    {
        try
        {
            if(!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }


            var itemToDelete = this.context.SupaliCustodians
                .Where(i => i.Custodian_ID == key)
                .Include(i => i.SupaliPatients)
                .FirstOrDefault();

            if (itemToDelete == null)
            {
                ModelState.AddModelError("", "Item no longer available");
                return BadRequest(ModelState);
            }

            this.OnSupaliCustodianDeleted(itemToDelete);
            this.context.SupaliCustodians.Remove(itemToDelete);
            this.context.SaveChanges();

            return new NoContentResult();
        }
        catch(Exception ex)
        {
            ModelState.AddModelError("", ex.Message);
            return BadRequest(ModelState);
        }
    }

    partial void OnSupaliCustodianUpdated(Models.SupAliDb.SupaliCustodian item);

    [HttpPut("{Custodian_ID}")]
    [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
    public IActionResult PutSupaliCustodian(int key, [FromBody]Models.SupAliDb.SupaliCustodian newItem)
    {
        try
        {
            if(!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            if (newItem == null || (newItem.Custodian_ID != key))
            {
                return BadRequest();
            }

            this.OnSupaliCustodianUpdated(newItem);
            this.context.SupaliCustodians.Update(newItem);
            this.context.SaveChanges();

            var itemToReturn = this.context.SupaliCustodians.Where(i => i.Custodian_ID == key);
            Request.QueryString = Request.QueryString.Add("$expand", "SupaliAgency,SupaliRelationshipType");
            return new ObjectResult(SingleResult.Create(itemToReturn));
        }
        catch(Exception ex)
        {
            ModelState.AddModelError("", ex.Message);
            return BadRequest(ModelState);
        }
    }

    [HttpPatch("{Custodian_ID}")]
    [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
    public IActionResult PatchSupaliCustodian(int key, [FromBody]Delta<Models.SupAliDb.SupaliCustodian> patch)
    {
        try
        {
            if(!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            var itemToUpdate = this.context.SupaliCustodians.Where(i => i.Custodian_ID == key).FirstOrDefault();

            if (itemToUpdate == null)
            {
                ModelState.AddModelError("", "Item no longer available");
                return BadRequest(ModelState);
            }

            patch.Patch(itemToUpdate);

            this.OnSupaliCustodianUpdated(itemToUpdate);
            this.context.SupaliCustodians.Update(itemToUpdate);
            this.context.SaveChanges();

            var itemToReturn = this.context.SupaliCustodians.Where(i => i.Custodian_ID == key);
            Request.QueryString = Request.QueryString.Add("$expand", "SupaliAgency,SupaliRelationshipType");
            return new ObjectResult(SingleResult.Create(itemToReturn));
        }
        catch(Exception ex)
        {
            ModelState.AddModelError("", ex.Message);
            return BadRequest(ModelState);
        }
    }

    partial void OnSupaliCustodianCreated(Models.SupAliDb.SupaliCustodian item);

    [HttpPost]
    [EnableQuery(MaxExpansionDepth=10,MaxAnyAllExpressionDepth=10,MaxNodeCount=1000)]
    public IActionResult Post([FromBody] Models.SupAliDb.SupaliCustodian item)
    {
        try
        {
            if(!ModelState.IsValid)
            {
                return BadRequest(ModelState);
            }

            if (item == null)
            {
                return BadRequest();
            }

            this.OnSupaliCustodianCreated(item);
            this.context.SupaliCustodians.Add(item);
            this.context.SaveChanges();

            var key = item.Custodian_ID;

            var itemToReturn = this.context.SupaliCustodians.Where(i => i.Custodian_ID == key);

            Request.QueryString = Request.QueryString.Add("$expand", "SupaliAgency,SupaliRelationshipType");

            return new ObjectResult(SingleResult.Create(itemToReturn))
            {
                StatusCode = 201
            };
        }
        catch(Exception ex)
        {
            ModelState.AddModelError("", ex.Message);
            return BadRequest(ModelState);
        }
    }
  }
}
